@@ -23,7 +23,9 @@ from utils import uncomment_or_update_or_append_line as uua_local
 
 FABSETUP_DIR = dirname(__file__)
 FABFILE_DATA_DIR = join(FABSETUP_DIR, 'fabfile-data')
+
 FABSETUP_CUSTOM_DIR = join(expanduser('~'), '.fabsetup-custom')
+FABSETUP_DOWNLOADS_DIR = join(expanduser('~'), '.fabsetup-downloads')
 
 
 def suggest_localhost(func):
@@ -100,42 +102,60 @@ def needs_repo_fabsetup_custom(func):
 
     @wraps(func)
     def wrapper(*args, **kwargs):
-        custom_dir = FABSETUP_CUSTOM_DIR
-        presetting_dir = join(FABFILE_DATA_DIR, 'presetting-fabsetup-custom')
-        if not isdir(custom_dir):
-            print(yellow('\n** **     Init ') +
-                  yellow('~/.fabsetup-custom', bold=True) +
-                  yellow('      ** **\n'))
-            print(yellow(flo('** Create files in dir {custom_dir} **')))
-            local(flo('mkdir -p {custom_dir}'))
-            local(flo('cp -r --no-clobber {presetting_dir}/. {custom_dir}'))
-            import_fabsetup_custom(globals())
-        else:
-            with quiet():
-                local(flo('cp -r --no-clobber {presetting_dir}/. {custom_dir}'))
+        if not os.path.exists(FABSETUP_CUSTOM_DIR):
+            msg = '''\
+Git repository ~/.fabsetup-custom with configurations does not exist.
+This configs are required to use fabsetup.
 
-        if not isdir(join(custom_dir, '.git')):
-            print(yellow(
-                '\n** Git repo ~/.fabsetup-custom: init and first commit **'))
-            local(flo('cd {custom_dir} && git init'))
-            local(flo('cd {custom_dir} && git add .'))
-            local(flo('cd {custom_dir} && git commit -am "Initial commit"'))
-            print(yellow("** Done. Don't forget to create a backup of your "
-                         '~/.fabsetup-custom repo **\n'))
-            print(yellow("** But do not make it public, it's custom **\n",
-                         bold=True))
-        else:
-            with quiet():
-                cmd = flo('cd {custom_dir} && git status --porcelain')
-                res = local(cmd, capture=True)
-                if res:
-                    print(yellow('\n** git repo  ') +
-                          magenta('~/.fabsetup-custom  ') +
-                          yellow('has uncommitted changes: **'))
-                    print(cmd)
-                    print(yellow(res, bold=True))
-                    print(yellow("** Don't forget to commit them and make a "
-                                 "backup of your repo **\n"))
+Clone it if you already have your own fabsetup-custom repository:
+
+    git clone <user>@<hostname>:/path/to/fabsetup-custom.git  ~/.fabetup-custom
+
+Else, initialize a new repository.
+
+Init a new repository `~/.fabsetup-custom`?'''
+            if not query_yes_no(msg, default='yes'):
+                sys.exit('abort')
+            custom_dir = FABSETUP_CUSTOM_DIR
+            presetting_dir = join(FABFILE_DATA_DIR,
+                                  'presetting-fabsetup-custom')
+            if not isdir(custom_dir):
+                print(yellow('\n** **     Init ') +
+                      yellow('~/.fabsetup-custom', bold=True) +
+                      yellow('      ** **\n'))
+                print(yellow(flo('** Create files in dir {custom_dir} **')))
+                local(flo('mkdir -p {custom_dir}'))
+                local(flo('cp -r --no-clobber {presetting_dir}/. {custom_dir}'))
+                import_fabsetup_custom(globals())
+            else:
+                with quiet():
+                    local(flo(
+                        'cp -r --no-clobber {presetting_dir}/. {custom_dir}'))
+
+            if not isdir(join(custom_dir, '.git')):
+                print(yellow(
+                    '\n** Git repo ~/.fabsetup-custom: '
+                    'init and first commit **'))
+                local(flo('cd {custom_dir} && git init'))
+                local(flo('cd {custom_dir} && git add .'))
+                local(flo('cd {custom_dir} && git commit -am "Initial commit"'))
+                print(yellow("** Done. Don't forget to create a backup of your "
+                             '~/.fabsetup-custom repo **\n'))
+                print(yellow("** But do not make it public, it's custom **\n",
+                             bold=True))
+            else:
+                with quiet():
+                    cmd = flo('cd {custom_dir} && git status --porcelain')
+                    res = local(cmd, capture=True)
+                    if res:
+                        print(yellow('\n** git repo  ') +
+                              magenta('~/.fabsetup-custom  ') +
+                              yellow('has uncommitted changes: **'))
+                        print(cmd)
+                        print(yellow(res, bold=True))
+                        print(yellow(
+                            "** Don't forget to commit them and make a "
+                            "backup of your repo **\n"))
         return func(*args, **kwargs)
     return wrapper
 
@@ -300,10 +320,9 @@ def install_package(package):
     install_packages([package])
 
 
-# TODO: base_dir must be compliant to addons
 @needs_packages('git')
-def checkup_git_repos(repos, base_dir='~/repos',
-                      verbose=False, prefix='', postfix=''):
+def checkup_git_repos_legacy(repos, base_dir='~/repos',
+                             verbose=False, prefix='', postfix=''):
     '''Checkout or update git repos.
 
     repos must be a list of dicts each with an url and optional with a name
@@ -312,14 +331,13 @@ def checkup_git_repos(repos, base_dir='~/repos',
     run(flo('mkdir -p {base_dir}'))
     for repo in repos:
         cur_base_dir = repo.get('base_dir', base_dir)
-        checkup_git_repo(url=repo['url'], name=repo.get('name', None),
-                         base_dir=cur_base_dir, verbose=verbose,
-                         prefix=prefix, postfix=postfix)
+        checkup_git_repo_legacy(url=repo['url'], name=repo.get('name', None),
+                                base_dir=cur_base_dir, verbose=verbose,
+                                prefix=prefix, postfix=postfix)
 
 
-# TODO: base_dir must be compliant to addons
-def checkup_git_repo(url, name=None, base_dir='~/repos',
-                     verbose=False, prefix='', postfix=''):
+def checkup_git_repo_legacy(url, name=None, base_dir='~/repos',
+                            verbose=False, prefix='', postfix=''):
     '''Checkout or update a git repo.'''
     if not name:
         match = re.match(r'.*/(.+)\.git', url)
@@ -344,6 +362,112 @@ def checkup_git_repo(url, name=None, base_dir='~/repos',
     return name
 
 
+AddonPackage = utlz.namedtuple(
+    typename='Names',
+    field_names=['module_dir'],
+    lazy_vals={
+        # eg. 'fabsetup_theno_termdown'
+        'module_name': lambda self: self.module_dir.rsplit('/', 1)[-1],
+
+        # eg. 'fabsetup-theno-termdown'
+        'package_name': lambda self: self.module_name.replace('_', '-'),
+        # would be 'site-packages' when fabsetup is installed as pip package:
+        # 'package_name': lambda self: self.module_dir.rsplit('/', 2)[-2],
+
+        # eg. '/home/theno/.fabsetup-addon-repos/fabsetup-theno-termdown'
+        'package_dir': lambda self: dirname(self.module_dir),
+
+        # '/home/theno/.fabsetup-addon-repos/fabsetup-theno-termdown/fabsetup_theno_termdown/files'
+        'default_files_basedir': lambda self: join(self.module_dir, 'files'),
+
+        # eg. '/home/theno/.fabsetup-custom/fabsetup-theno-termdown'
+        'custom_dir': lambda self: join(FABSETUP_CUSTOM_DIR,
+                                        self.package_name),
+
+        # eg. '/home/theno/.fabsetup-custom/fabsetup-theno-termdown/config.py'
+        'custom_config': lambda self: join(self.custom_dir, 'config.py'),
+
+        # eg. '/home/theno/.fabsetup-custom/fabsetup-theno-termdown/files'
+        'custom_files_basedir': lambda self: join(FABSETUP_CUSTOM_DIR,
+                                                  self.package_name, 'files'),
+
+        # eg. '/home/theno/.fabsetup-downloads/fabsetup-theno-termdown'
+        'downloads_basedir': lambda self: join(FABSETUP_DOWNLOADS_DIR,
+                                               self.package_name),
+    })
+
+
+def checkup_git_repos_wrapper(package):
+
+    def checkup_git_repos(repos, base_dir=package.downloads_basedir,
+                          verbose=True, prefix='', postfix=''):
+        '''Checkout or update git repos.
+
+        repos must be a list of dicts each with an url and optional with a name
+        value.
+
+        Example:
+
+            checkup_git_repos(
+                repos=[
+                    {
+                        'url': 'https://github.com/theno/utlz.git',
+                    },
+                    {
+                        'url': 'https://github.com/theno/ctutlz.git',
+                        'name': 'ct-utils',
+                    },
+                ],
+                prefix='\n### ', postfix='\n')
+        '''
+        for repo in repos:
+            cur_base_dir = repo.get('base_dir', base_dir)
+            checkup_git_repo_legacy(url=repo['url'],
+                                    name=repo.get('name', None),
+                                    base_dir=cur_base_dir, verbose=verbose,
+                                    prefix=prefix, postfix=postfix)
+
+    return checkup_git_repos
+
+
+def checkup_git_repo_wrapper(package):
+
+    @needs_packages('git')
+    def checkup_git_repo(url, name=None, base_dir=package.downloads_basedir,
+                         verbose=True, prefix='', postfix=''):
+        '''Checkout or update a git repo.'''
+        if not name:
+            # eg. url = 'https://github.com/my-repo-name.git'
+            # =>  name = 'my-repo-name'
+            match = re.match(r'.*/(.+)\.git', url)
+            assert match, flo("Unable to extract repo name from '{url}'")
+            name = match.group(1)
+        assert name is not None, flo(
+            'Cannot extract repo name from repo: {url}')
+        assert name != '', flo(
+            'Cannot extract repo name from repo: {url} (empty)')
+        if verbose:
+            name_blue = blue(name)
+            print_msg(flo('{prefix}Checkout or update {name_blue}{postfix}'))
+        filename_readme = '~/.fabsetup-downloads/README.md'
+        if not exists(filename_readme):
+            install_file_legacy(filename_readme)
+        if not exists(base_dir):
+            run(flo('mkdir -p {base_dir}'))
+        if not exists(flo('{base_dir}/{name}/.git')):
+            run(flo('  &&  '.join([
+                    'cd {base_dir}',
+                    'git clone  {url}  {name}'])),
+                msg='clone repo')
+        else:
+            if verbose:
+                print_msg('update: pull from origin')
+            run(flo('cd {base_dir}/{name}  &&  git pull'))
+        return flo('{base_dir}/{name}')
+
+    return checkup_git_repo
+
+
 def _install_file_from_template_legacy(from_template, to_, **substitutions):
     from_str = filled_out_template(from_template, **substitutions)
     with tempfile.NamedTemporaryFile(prefix='fabsetup') as tmp_file:
@@ -411,35 +535,6 @@ def install_user_command_legacy(command, **substitutions):
     run(flo('chmod 755 {path}'))
 
 
-AddonPackage = utlz.namedtuple(
-    typename='Names',
-    field_names=['module_dir'],
-    lazy_vals={
-        # fabsetup_theno_termdown
-        'module_name': lambda self: self.module_dir.rsplit('/', 1)[-1],
-
-        # fabsetup-theno-termdown
-        'package_name': lambda self: self.module_dir.rsplit('/', 2)[-2],
-
-        # /home/theno/.fabsetup-addon-repos/fabsetup-theno-termdown
-        'package_dir': lambda self: dirname(self.module_dir),
-
-        # /home/theno/.fabsetup-addon-repos/fabsetup-theno-termdown/fabsetup_theno_termdown/files
-        'default_files_basedir': lambda self: join(self.module_dir, 'files'),
-
-        # /home/theno/.fabsetup-custom/fabsetup-theno-termdown
-        'custom_dir': lambda self: join(FABSETUP_CUSTOM_DIR,
-                                        self.package_name),
-
-        # /home/theno/.fabsetup-custom/fabsetup-theno-termdown/config.py
-        'custom_config': lambda self: join(self.custom_dir, 'config.py'),
-
-        # /home/theno/.fabsetup-custom/fabsetup-theno-termdown/files
-        'custom_files_basedir': lambda self: join(FABSETUP_CUSTOM_DIR,
-                                                  self.package_name, 'files'),
-    })
-
-
 def _determine_froms(addon_package, path):
     '''Return 2-Tuple (from_custom, from_default) for a given path.
 
@@ -484,7 +579,8 @@ def _determine_froms(addon_package, path):
 def _substituted(string, substitutions):
     substituted = string
     for key, val in substitutions.items():
-        substituted = string.replace(key, val)
+        if isinstance(val, str):
+            substituted = substituted.replace(key, val)
     return substituted
 
 
@@ -534,7 +630,7 @@ def install_file_wrapper(addon_package):
 
             from_custom, from_default = _determine_froms(addon_package, path)
 
-            from_custom_template = flo('{from_default}.template')
+            from_custom_template = flo('{from_custom}.template')
             from_default_template = flo('{from_default}.template')
 
             if isfile(from_custom):
@@ -735,5 +831,72 @@ def dn_cn_of_certificate_with_san(domain):
     return cn_dn
 
 
+def extract_minors_from_setup_py(filename_setup_py):
+    '''Extract supported python minor versions from setup.py and return them
+    as a list of str.
+
+    Return example:
+
+        ['2.6', '2.7', '3.3', '3.4', '3.5', '3.6']
+    '''
+    # eg: minors_str = '2.6\n2.7\n3.3\n3.4\n3.5\n3.6'
+    minors_str = fabric.api.local(
+        flo('grep --perl-regexp --only-matching '
+            '"(?<=Programming Language :: Python :: )\\d+\\.\\d+" '
+            '{filename_setup_py}'),
+        capture=True)
+    # eg: minors = ['2.6', '2.7', '3.3', '3.4', '3.5', '3.6']
+    minors = minors_str.split()
+    return minors
+
+
+def determine_latest_pythons(minors):
+    '''Determine latest stable python versions and return them as a list of str.
+
+    Args:
+        minors([<str>,..]): List of python minor versions as str, eg.
+                            ['2.6', '2.7', '3.3', '3.4', '3.5', '3.6']
+
+    Return example:
+
+        ['2.6.9', '2.7.14', '3.3.7', '3.4.8', '3.5.5', '3.6.4']
+    '''
+    # eg: ['2.6.9', '2.7.14', '3.3.7', '3.4.8', '3.5.5', '3.6.4']
+    latests = []
+
+    versions_str = fabric.api.local(flo(
+        'pyenv install --list | tr -d [:blank:] | '
+        'grep -P "^[\d\.]+$"'), capture=True)
+    versions = versions_str.split()
+
+    for minor in minors:
+        candidates = [version
+                      for version
+                      in versions
+                      if version.startswith(minor)]
+        # sort version numbers: https://stackoverflow.com/a/2574090
+        candidates.sort(key=lambda s: [int(u) for u in s.split('.')])
+        latest = candidates[-1]
+        latests.append(latest)
+
+    print(latests)
+    return latests
+
+
+def highest_minor(python_versions):
+    '''Return highest minor of a list of stable (semantic) versions.
+
+    Example:
+        >>> python_versions = [
+        ...     '2.6.9', '2.7.14', '3.3.7', '3.4.8', '3.5.5', '3.6.4']
+        >>> highest_minor(python_versions)
+        '3.6'
+
+    '''
+    highest = python_versions[-1]
+    major, minor, patch = highest.split('.', 2)
+    return flo('{major}.{minor}')
+
+
 if __name__ == '__main__':
     doctest.testmod()
